;; Generated package description from excorporate.el  -*- no-byte-compile: t -*-
(define-package "excorporate" "1.1.3.0.20250627.211849" "Exchange Web Services (EWS) integration" '((fsm "0.2.1") (soap-client "3.2.0") (url-http-ntlm "2.0.6") (url-http-oauth "0.8.4")) :commit "d0dc6a28a0b39b67b01c63163217f8ebd055df4d" :authors '(("Thomas Fitzsimmons" . "fitzsim@fitzsim.org")) :maintainer '("Thomas Fitzsimmons" . "fitzsim@fitzsim.org") :keywords '("calendar") :url "https://www.fitzsim.org/blog/")
